using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

static class Program
{ static void Main() { Game1 game = new Game1(); game.Run(); }

public class Game1 : Microsoft.Xna.Framework.Game
{ private GraphicsDeviceManager g;
  private Model model;
  private BasicEffect effect;

  private ButtonState[] bs = new ButtonState[12]; //8 Buttons + 4 DPads
  private Vector2    [] ts = new Vector2[3];      //3 Thumbsticks with x/y
  private float positionX = 0.0f, positionY = 0.0f, positionZ = 0.0f;
  private float scaleX    = 1.0f, scaleY    = 1.0f, scaleZ    = 1.0f;
  private float rotationX = 0.0f, rotationY = 0.0f, rotationZ = 0.0f;
  public  Form1 form;
  public Game1() { g = new GraphicsDeviceManager( this ); }
  protected override void Initialize()
  { g.PreferredBackBufferWidth  = 600;
    g.PreferredBackBufferHeight = 600;
    g.ApplyChanges();
    g.IsFullScreen = false;
    Window.AllowUserResizing = true;
    Window.Title = "XNA Using NoName GamePads";
    base.Initialize();
    form = new Form1();
    form.Location = new System.Drawing.Point( Window.ClientBounds.Right+5, Window.ClientBounds.Top );
    form.Size     = new System.Drawing.Size ( 100, Window.ClientBounds.Height );
    form.Show();
  }
  protected override void LoadContent()
  { Content.RootDirectory = "Content";
    model = Content.Load< Model >( "tiger" );
    effect = (BasicEffect)model.Meshes[0].Effects[0];
    effect.View = Matrix.CreateLookAt( new Vector3(0.0f, 0.0f, 4.0f), Vector3.Zero, Vector3.Up );
    effect.Projection =
      Matrix.CreatePerspectiveFieldOfView( MathHelper.Pi/4, 1f, 0.1f, 1000.0f );
  }
  protected override void Update( GameTime gameTime )
  { if ( Soopah.Xna.Input.DirectInputGamepad.Gamepads.Count < 1 )
    { System.Windows.Forms.MessageBox.Show( "Error: Can't find a GamePad" ); Exit(); }
    for ( int i=0; i < 8; i++ ) //check all possible 8 Buttons, 4 DPads, 3 ThumbSticks
      bs[i] = Soopah.Xna.Input.DirectInputGamepad.Gamepads[0].Buttons.List[i];
    bs [ 8] = Soopah.Xna.Input.DirectInputGamepad.Gamepads[0].DPad.Down;
    bs [ 9] = Soopah.Xna.Input.DirectInputGamepad.Gamepads[0].DPad.Up;
    bs [10] = Soopah.Xna.Input.DirectInputGamepad.Gamepads[0].DPad.Left;
    bs [11] = Soopah.Xna.Input.DirectInputGamepad.Gamepads[0].DPad.Right;
    ts [ 0] = Soopah.Xna.Input.DirectInputGamepad.Gamepads[0].ThumbSticks.Left;
    ts [ 1] = Soopah.Xna.Input.DirectInputGamepad.Gamepads[0].ThumbSticks.Right;
    ts [ 2] = Soopah.Xna.Input.DirectInputGamepad.Gamepads[0].ThumbSticks.Third;
    ButtonState bp = ButtonState.Pressed;
    for ( int i=0; i < 12; i++ )
      if ( bs[i] == bp ) form.checkbox[i].Checked = true;
      else               form.checkbox[i].Checked = false;
    if ( bs[ 0] == bp ) scaleY    *= 1.01f; //Button No. 1
    if ( bs[ 1] == bp ) scaleX    *= 1.01f; //Button No. 2
    if ( bs[ 2] == bp ) scaleY    *= 0.99f; //Button No. 3
    if ( bs[ 3] == bp ) scaleX    *= 0.99f; //Button No. 4
    if ( bs[ 4] == bp ) scaleZ    *= 0.99f; //Button No. 5
    if ( bs[ 5] == bp ) scaleZ    *= 1.01f; //Button No. 6
    if ( bs[ 6] == bp ) rotationZ += 0.01f; //Button No. 7
    if ( bs[ 7] == bp ) rotationZ -= 0.01f; //Button No. 8
    if ( bs[ 8] == bp ) positionY -= 0.01f; //DPad.Down
    if ( bs[ 9] == bp ) positionY += 0.01f; //DPad.Up
    if ( bs[10] == bp ) positionX -= 0.01f; //DPad.Left
    if ( bs[11] == bp ) positionX += 0.01f; //DPad.Right
    for ( int i=0, j=0; i < 3; i++, j+=2 ) //3 ThumbSticks mapped on 6 TrackBars
    { if ( Math.Abs(ts[i].X) < 0.2f ) ts[i].X = 0f; //suppress small movements
      if ( Math.Abs(ts[i].Y) < 0.2f ) ts[i].Y = 0f; //suppress small movements
      form.trackbar[j  ].Value = (int)( 50f * (ts[i].X+1f) ); //map to range 0..100
      form.trackbar[j+1].Value = (int)( 50f * (ts[i].Y+1f) ); //map to range 0..100
    }
    rotationX -= ts[0].X / 50f;
    rotationY -= ts[1].Y / 50f;
    rotationZ -= ts[2].X / 50f;
    base.Update( gameTime );
  }
  protected override void Draw( GameTime gameTime )
  { g.GraphicsDevice.Clear( Color.DarkBlue );
    effect.EnableDefaultLighting();
    effect.World = Matrix.CreateScale( scaleX, scaleY, scaleZ ) *
                   Matrix.CreateRotationX( rotationX ) *
                   Matrix.CreateRotationY( rotationY ) *
                   Matrix.CreateRotationZ( rotationZ ) *
                   Matrix.CreateTranslation( positionX, positionY, positionZ );
    model.Meshes[0].Draw();
  }
} // end of class Game1
} // end of class Program